/*
qpick.c  Version 0.3.0  Powerball Quick Pick Drawings
Copyright (C) 2002-2010   dondalah721@yahoo.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

/*
Subroutine call:
    	int select;
    	int pwrball;
    	double odds;
    	double getpb(int select, int pwrball);
    	odds = getpb(select,pwrball);
*/

/* If the chi-square test fails, repeat your test. */
/* The chi-square usually passes in the next test. */
/* If the chi-square fails consistently, then the */
/* program has to be reviewed for correctness. */

/* The higher the number of iterations, the more */
/* likely the chi-square test will pass.  The chi-square */
/* test needs to have its categories filled with at */
/* least a 10 count in order to pass consistently. */
/* With 100000 iterations, the chi-square test passes */
/* consistently. */

/* See the documentation in pbprob.c regarding the */
/* calls to getpb.c. */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "rnd.h"

#define PWRBALL 42
#define WHITEBALL 53
#define FIVE 5

void putstx(pgm)
char *pgm;
   {
   fprintf(stderr,"Usage: %s iterations\n", pgm);
   fprintf(stderr,"Where iterations is 1 - 999999999\n");
   fprintf(stderr,"Example: %s 100000\n", pgm);
   exit(1);
   } /* putstx */

int main(argc,argv)
int argc;
char **argv;
   {
   int i,maxwhball,indx;
   unsigned char *p,*q,*r;
   unsigned char *gameptr;
   unsigned char *trialptr;
   unsigned char whiteball[64];
   unsigned char winners[64];
   unsigned char game[16];
   unsigned char trial[16];
   int tmp;
   int gamepb;
   int winpb;
   int totmtch,won;
   int totplay;
   int maxplay;
   double prob,dblplay;
   double winning,totprize,netearn;
   double *prizeptr;
   double prize[16];
   double odds,sum,totbingo,chisq;
   double totprob;
   double expected;
   double getpb(int select, int pwrball);
   double *bingoptr;
   double bingo[16];
   if (argc != 2) putstx(*argv);
   maxplay = atoi(*(argv+1));
   if (maxplay < 1 || maxplay > 999999999)
      {
      fprintf(stderr,"Invalid iterations\n");
      putstx(*argv);
      }
   sd = (unsigned char *) rndinit();
   prizeptr = (double *) prize;
   *prizeptr++ = 5000000.00;
   *prizeptr++ = 100000.00;
   *prizeptr++ = 5000.00;
   *prizeptr++ = 100.00;
   *prizeptr++ = 100.00;
   *prizeptr++ = 7.00;
   *prizeptr++ = 7.00;
   *prizeptr++ = 4.00;
   *prizeptr   = 3.00;
   p = (unsigned char *) whiteball;
   q = (unsigned char *) whiteball + WHITEBALL;
   i = 0;
   while (p < q)
      *p++ = (unsigned char) ++i;
   *p = (unsigned char) '\0';
   p = (unsigned char *) winners;
   q = (unsigned char *) winners + WHITEBALL;
   while (p < q)
      *p++ = (unsigned char) '\0';
   p = (unsigned char *) whiteball;
   q = (unsigned char *) whiteball + WHITEBALL - 1;
   r = (unsigned char *) winners;
   gameptr = (unsigned char *) game;
   i = 0;
   maxwhball = WHITEBALL;
   while (i < FIVE)
      {
      indx = rndnum(maxwhball,sd);
      tmp = *(p+indx);
      *(r+tmp)   = (unsigned char) 255;
      *gameptr++ = (unsigned char) tmp;
      *(p+indx) = *q;
      *q-- = '\0';
      maxwhball--;
      i++;
      } /* pick 5 balls without replacement */
   gamepb = rndnum(PWRBALL,sd) + 1;
   printf("Powerball:  ");
   gameptr = (unsigned char *) game;
   i = FIVE;
   while (i--)
      {
      printf("%2d ", (int) *gameptr++);
      } /* print 5 winning balls */
   printf("- %2d\n\n", gamepb);
   printf("Match      Wins          Prize    "
      "   Expected           Odds\n");
   bingoptr = (double *) bingo;
   i = 9;
   while (i--) *bingoptr++ = 0.0;
   bingoptr = (double *) bingo;
   totplay = maxplay;
   while (totplay--)
      {
      p = (unsigned char *) whiteball;
      q = (unsigned char *) whiteball + WHITEBALL;
      i = 0;
      while (p < q)
         *p++ = (unsigned char) ++i;
      *p = (unsigned char) '\0';
      totmtch = winpb = 0;
      p = (unsigned char *) whiteball;
      q = (unsigned char *) whiteball + WHITEBALL - 1;
      r = (unsigned char *) winners;
      trialptr = (unsigned char *) trial;
      maxwhball = WHITEBALL;
      i = FIVE;
      while (i--)
         {
         indx = rndnum(maxwhball,sd);
         tmp = (int) *(p+indx);
         *trialptr++ = (unsigned char) tmp;
         if (*(r+tmp) == 255) totmtch++;
         *(p+indx) = *q;
         *q-- = '\0';
         maxwhball--;
         } /* pick 5 balls without replacement */
      winpb = rndnum(PWRBALL,sd) + 1;
      if (winpb != gamepb) winpb = 0;
      won = 999999;
      if (totmtch == FIVE)
         {
         if (winpb) won = 0;
         else won = 1;
         } /* select 5 */
      else if (totmtch == 4)
         {
         if (winpb) won = 2;
         else won = 3;
         } /* select 4 */
      else if (totmtch == 3)
         {
         if (winpb) won = 4;
         else won = 5;
         } /* select 3 */
      else if (totmtch == 2 && winpb)
         won = 6;
      else if (totmtch == 1 && winpb)
         won = 7;
      else if (totmtch == 0 && winpb)
         won = 8;
      else won = 9;
      *(bingoptr+won) += 1.0;
      } /* tst loop maxplay times */

   bingoptr = (double *) bingo;
   prizeptr = (double *) prize;
   dblplay = (double) maxplay;
   totprize = 0.0;

   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   /* prob = 54979155.0; */
   prob = getpb(5,1);
   expected = dblplay / prob;
   printf("5 1   %9.0f  %13.2f  %13.5f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   /* prob = 1249527.0; */
   prob = getpb(5,0);
   expected = dblplay / prob;
   printf("5 0   %9.0f  %13.2f  %13.5f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   /* prob = 274896.0; */
   prob = getpb(4,1);
   expected = dblplay / prob;
   printf("4 1   %9.0f  %13.2f  %13.5f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   /* prob = 6248.0; */
   prob = getpb(4,0);
   expected = dblplay / prob;
   printf("4 0   %9.0f  %13.2f  %13.2f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   /* prob = 7049.0; */
   prob = getpb(3,1);
   expected = dblplay / prob;
   printf("3 1   %9.0f  %13.2f  %13.2f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   /* prob = 161.0; */
   prob = getpb(3,0);
   expected = dblplay / prob;
   printf("3 0   %9.0f  %13.2f  %13.2f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   /* prob = 557.0; */
   prob = getpb(2,1);
   expected = dblplay / prob;
   printf("2 1   %9.0f  %13.2f  %13.2f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   /* prob = 121.0; */
   prob = getpb(1,1);
   expected = dblplay / prob;
   printf("1 1   %9.0f  %13.2f  %13.2f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   /* prob = 84.0; */
   prob = getpb(0,1);
   expected = dblplay / prob;
   printf("0 1   %9.0f  %13.2f  %13.2f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   bingoptr = (double *) bingo;
   sum = totbingo = totprob = 0.0;
   odds = getpb(5,1);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   odds = getpb(5,0);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   odds = getpb(4,1);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   odds = getpb(4,0);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   odds = getpb(3,1);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   odds = getpb(3,0);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   odds = getpb(2,1);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   odds = getpb(1,1);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   odds = getpb(0,1);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;

   sum += ((dblplay - totbingo)
      * (dblplay - totbingo)
      / (1.0 - totprob));

   expected = dblplay / (1.0 / (1.0 - totprob));
   printf("0 0   %9.0f  %13.2f  %13.2f   %21.10f\n",
      (dblplay - totbingo),
      0.0,
      expected,
      (1.0 / (1.0 - totprob)));
   printf("        -------      ---------\n");
   printf("Total %9.0f  %13.2f\n\n",
      dblplay, totprize);
   printf("Total Cost       %13.2f\n\n",
      dblplay);
   netearn = totprize - dblplay;
   printf("Net Income       %13.2f\n\n",
      netearn);
   chisq = (sum / dblplay) - dblplay;
   printf("Chi-square %f\n", chisq);
   printf("Range at 95 percent "
      "is 2.70039 to 19.0228\n");
   free(sd);
   return(0);
   } /* main */
